import React, { useState, useEffect, useRef, useMemo, Fragment, useCallback, useImperativeHandle } from 'react';
import styled, { keyframes, css, ThemeProvider } from 'styled-components';
import { areEqual, FixedSizeList } from 'react-window';

var DefaultThemeObj = Object.freeze({
  color: {
    border: '#ced4da',
    danger: '#dc3545',
    primary: '#007bff',
    disabled: '#e9ecef',
    placeholder: '#6E7276',
    dangerLight: 'rgba(220, 53, 69, 0.25)'
  },
  select: {},
  loader: {
    size: '0.625rem',
    padding: '0.375rem 0.75rem',
    color: 'rgba(0, 123, 255, 0.42)'
  },
  icon: {
    color: '#cccccc',
    padding: '0.5rem 0.9375rem',
    hoverColor: '#A6A6A6',
    clear: {
      fontWeight: 900,
      fontSize: '0.85em',
      transition: 'color 0.15s ease-in-out',
      fontFamily: '"Helvetica", "Arial", sans-serif'
    },
    caret: {
      size: '7px',
      transition: 'transform 0.225s ease-in-out, color 0.15s ease-in-out'
    }
  },
  control: {
    minHeight: '38px',
    borderWidth: '1px',
    borderStyle: 'solid',
    borderRadius: '0.25rem',
    boxShadow: '0 0 0 0.2rem',
    padding: '0.375rem 0.75rem',
    boxShadowColor: 'rgba(0, 123, 255, 0.25)',
    focusedBorderColor: 'rgba(0, 123, 255, 0.75)',
    transition: 'box-shadow 0.15s ease-in-out, border-color 0.15s ease-in-out'
  },
  menu: {
    width: '100%',
    margin: '0.5rem 0',
    padding: '0.15rem 0',
    borderRadius: '0.25rem',
    backgroundColor: '#fff',
    boxShadow: '0 0 0 1px rgba(0, 0, 0, 0.1), 0 4px 11px rgba(0, 0, 0, 0.1)',
    option: {
      textAlign: 'left',
      selectedColor: '#fff',
      selectedBgColor: '#007bff',
      padding: '0.375rem 0.75rem',
      focusedBgColor: 'rgba(0, 123, 255, 0.20)'
    }
  },
  noOptions: {
    fontSize: '1.25rem',
    margin: '0.25rem 0',
    color: 'hsl(0, 0%, 60%)',
    padding: '0.375rem 0.75rem'
  },
  multiValue: {
    margin: '1px 2px',
    borderRadius: '0.25rem',
    backgroundColor: '#e7edf3',
    label: {
      fontSize: '0.825em',
      borderRadius: '0.25rem',
      padding: '1px 2px 1px 6px'
    },
    clear: {
      fontWeight: 700,
      padding: '0 7px',
      fontSize: '0.675em',
      alignItems: 'center',
      borderRadius: '0.25rem',
      transition: 'color 0.15s ease-in-out, background-color 0.15s ease-in-out'
    }
  }
});

const FADE_IN_KEYFRAMES = keyframes`
  from {
    opacity: 0;
  } to {
    opacity: 1;
  }
`;
const FADE_IN_ANIMATION_CSS = css`
  animation: ${FADE_IN_KEYFRAMES} 0.225s ease-in-out forwards;
`;

var FilterMatchEnum;

(function (FilterMatchEnum) {
  FilterMatchEnum["ANY"] = "any";
  FilterMatchEnum["START"] = "start";
})(FilterMatchEnum || (FilterMatchEnum = {}));

const ValueIndexEnum = Object.freeze({
  NEXT: 0,
  PREVIOUS: 1
});
const OptionIndexEnum = Object.freeze({
  UP: 0,
  DOWN: 1,
  FIRST: 2,
  LAST: 3
});

const useDebounce = (value, delay) => {
  const [debouncedValue, setDebouncedValue] = useState(value);
  useEffect(() => {
    if (delay === undefined) {
      return;
    }

    const handler = setTimeout(() => {
      setDebouncedValue(value);
    }, delay);
    return () => {
      clearTimeout(handler);
    };
  }, [value, delay]);
  return delay === undefined ? value : debouncedValue;
};

const useIsFirstRender = () => {
  const isFirst = useRef(true);

  if (isFirst.current) {
    isFirst.current = false;
    return true;
  }

  return isFirst.current;
};

const useUpdateEffect = (effect, deps) => {
  const isFirstRender = useIsFirstRender();
  useEffect(() => {
    !isFirstRender && effect();
  }, deps);
};

const MENU_ITEM_SIZE_DEFAULT = 35;
const MENU_MAX_HEIGHT_DEFAULT = 300;
const NO_OPTIONS_MSG_DEFAULT = 'No options';
const PLACEHOLDER_DEFAULT = 'Select option..';
const FOCUSED_MULTI_DEFAULT = null;
const ON_CHANGE_SINGLE_VALUE_DEFAULT = null;
const OPTIONS_DEFAULT = [];
const SELECTED_OPTION_DEFAULT = [];
const FOCUSED_OPTION_DEFAULT = Object.freeze({
  index: -1
});

const TRIM_REGEXP = /^\s+|\s+$/g;
const OVERFLOW_REGEXP = /(auto|scroll)/;
const DIACRITICS_REGEXP = /[\u0300-\u036f]/g;
const MS_BROWSER_REGEXP = /MSIE |Trident\/|Edge\//;

function stripDiacritics(value) {
  return value.normalize('NFD').replace(DIACRITICS_REGEXP, '');
}

function isDocumentElement(el) {
  return el === document.documentElement || el === document.body || el === window;
}

function easeOutCubic(t, s, c, d) {
  return c * ((t = t / d - 1) * t * t + 1) + s;
}

function getScrollTop(el) {
  return isDocumentElement(el) ? window.pageYOffset : el.scrollTop;
}

function scrollTo(el, top) {
  if (isDocumentElement(el)) {
    window.scrollTo(0, top);
  } else {
    el.scrollTop = top;
  }
}

function getScrollParent(el) {
  let style = getComputedStyle(el);
  const excludeStaticParent = style.position === 'absolute';

  if (style.position === 'fixed') {
    return document.documentElement;
  }

  for (let parent = el; parent = parent ? parent.parentElement : null;) {
    style = getComputedStyle(parent);

    if (excludeStaticParent && style.position === 'static') {
      continue;
    } else if (OVERFLOW_REGEXP.test(`${style.overflow}${style.overflowY}${style.overflowX}`)) {
      return parent;
    }
  }

  return document.documentElement;
}

function smoothScrollTo(element, to, duration = 300, callback) {
  let currentTime = 0;
  const start = getScrollTop(element);
  const change = to - start;

  function smoothScroller() {
    currentTime += 5;
    scrollTo(element, easeOutCubic(currentTime, start, change, duration));

    if (currentTime < duration) {
      window.requestAnimationFrame(smoothScroller);
    } else {
      callback && callback();
    }
  }

  window.requestAnimationFrame(smoothScroller);
}

function isArrayWithLength(test) {
  return Array.isArray(test) && !!test.length;
}
function isPlainObject(test) {
  return test && typeof test === 'object' && !Array.isArray(test);
}
const isTouchDevice = () => window.matchMedia('(pointer: coarse)').matches;
const isEdgeOrIE = () => MS_BROWSER_REGEXP.test(window.navigator.userAgent);
function trimAndFormatFilterStr(value, filterIgnoreCase, filterIgnoreAccents) {
  let formatVal = value.replace(TRIM_REGEXP, '');

  if (filterIgnoreCase) {
    formatVal = formatVal.toLowerCase();
  }

  return !filterIgnoreAccents ? formatVal : stripDiacritics(formatVal);
}
function mergeDeep(target, source) {
  const output = Object.assign({}, target);
  Object.keys(source).forEach(key => {
    if (isPlainObject(source[key])) {
      output[key] = !(key in target) ? source[key] || '' : mergeDeep(target[key], source[key]);
    } else {
      output[key] = source[key];
    }
  });
  return output;
}
function scrollMenuIntoViewOnOpen(menuEl, menuScrollDuration, scrollMenuIntoView, handleOnMenuOpen) {
  if (!scrollMenuIntoView || !menuEl || !menuEl.getBoundingClientRect) {
    handleOnMenuOpen();
    return;
  }

  const {
    top: menuTop,
    bottom: menuBottom,
    height: menuHeight
  } = menuEl.getBoundingClientRect();
  const viewHeight = window.innerHeight;
  const viewSpaceBelow = viewHeight - menuTop;

  if (viewSpaceBelow >= menuHeight) {
    handleOnMenuOpen();
    return;
  }

  const scrollParent = getScrollParent(menuEl);
  const scrollTop = getScrollTop(scrollParent);
  const scrollSpaceBelow = scrollParent.getBoundingClientRect().height - scrollTop - menuTop;

  if (scrollSpaceBelow < menuHeight) {
    handleOnMenuOpen(scrollSpaceBelow);
    return;
  }

  const marginBottom = parseInt(getComputedStyle(menuEl).marginBottom || '0', 10);
  const scrollDown = menuBottom - viewHeight + scrollTop + marginBottom;
  smoothScrollTo(scrollParent, scrollDown, menuScrollDuration, handleOnMenuOpen);
}
function validateSetValueParam(values, menuOptions, getOptionValueCB) {
  if (values === null || values === undefined) {
    return SELECTED_OPTION_DEFAULT;
  }

  const validValuesArr = normalizeValue(values).filter(x => isPlainObject(x)).map(x => getOptionValueCB(x)).filter((item, index, self) => self.indexOf(item) === index);

  if (!isArrayWithLength(validValuesArr)) {
    return SELECTED_OPTION_DEFAULT;
  }

  const results = [];

  for (const option of menuOptions) {
    if (validValuesArr.includes(getOptionValueCB(option))) {
      results.push(option);

      if (validValuesArr.length === results.length) {
        break;
      }
    }
  }

  return results;
}
function normalizeValue(value, getOptionValueCB, getOptionLabelCB) {
  const initialValues = Array.isArray(value) ? value.filter(Boolean) : isPlainObject(value) ? [value] : SELECTED_OPTION_DEFAULT;

  if (!getOptionValueCB || !getOptionLabelCB || !isArrayWithLength(initialValues)) {
    return initialValues;
  }

  return initialValues.map(v => ({
    data: v,
    value: getOptionValueCB(v),
    label: getOptionLabelCB(v)
  }));
}

const useMenuHeight = (menuRef, menuOpen, menuHeightDefault, menuScrollDuration, scrollMenuIntoView, onMenuOpen, onMenuClose) => {
  const resetMenuHeight = useRef(false);
  const [menuHeight, setMenuHeight] = useState(menuHeightDefault);
  useUpdateEffect(() => {
    if (menuOpen) {
      const handleOnMenuOpen = availableSpace => {
        onMenuOpen && onMenuOpen();

        if (availableSpace) {
          resetMenuHeight.current = true;
          setMenuHeight(availableSpace);
        }
      };

      scrollMenuIntoViewOnOpen(menuRef.current, menuScrollDuration, scrollMenuIntoView, handleOnMenuOpen);
    } else {
      onMenuClose && onMenuClose();

      if (resetMenuHeight.current) {
        resetMenuHeight.current = false;
        setMenuHeight(menuHeightDefault);
      }
    }
  }, [menuRef, menuOpen, onMenuClose, onMenuOpen, menuHeightDefault, scrollMenuIntoView, menuScrollDuration]);
  return menuHeight;
};

const useMenuOptions = (options, debouncedInputValue, filterMatchFrom, hideSelectedOptions, selectedOption, getOptionValueCB, getOptionLabelCB, getIsOptionDisabled, getFilterOptionString, filterIgnoreCase, filterIgnoreAccents) => {
  const [menuOptions, setMenuOptions] = useState(OPTIONS_DEFAULT);
  useEffect(() => {
    const isFilterMatchAny = filterMatchFrom === FilterMatchEnum.ANY;
    const selectedValues = selectedOption.length ? selectedOption.map(x => x.value) : undefined;
    const normalizedInput = trimAndFormatFilterStr(debouncedInputValue, filterIgnoreCase, filterIgnoreCase);

    const getIsOptionDisabled2 = getIsOptionDisabled || (data => !!data.isDisabled);

    const getFilterOptionString2 = getFilterOptionString || (option => String(option.label));

    const isOptionFilterMatch = menuOption => {
      const normalizedOptionLabel = trimAndFormatFilterStr(getFilterOptionString2(menuOption), filterIgnoreCase, filterIgnoreCase);
      return isFilterMatchAny ? normalizedOptionLabel.indexOf(normalizedInput) > -1 : normalizedOptionLabel.substr(0, normalizedInput.length) === normalizedInput;
    };

    const parseMenuOption = data => {
      const value = getOptionValueCB(data);
      const menuOption = Object.assign({
        data,
        value,
        label: getOptionLabelCB(data)
      }, selectedValues && selectedValues.includes(value) && {
        isSelected: true
      });

      if (normalizedInput && !isOptionFilterMatch(menuOption) || hideSelectedOptions && menuOption.isSelected) {
        return;
      }

      return Object.assign({}, menuOption, {}, getIsOptionDisabled2(data) && {
        isDisabled: true
      });
    };

    const menuOptionsOrDefault = options.reduce((accumulator, data) => {
      const option = parseMenuOption(data);
      option && accumulator.push(option);
      return accumulator;
    }, []) || OPTIONS_DEFAULT;
    setMenuOptions(menuOptionsOrDefault);
  }, [options, selectedOption, hideSelectedOptions, filterMatchFrom, filterIgnoreCase, filterIgnoreAccents, debouncedInputValue, getFilterOptionString, getIsOptionDisabled, getOptionValueCB, getOptionLabelCB]);
  return menuOptions;
};

const attrPrefix = 'rfs';
const isNotProd = process.env.NODE_ENV !== 'production';
const IME_KEY_CODE = 229;
const INPUT_TAG_NAME = 'INPUT';
const MOUSE_DOWN_EVENT_TYPE = 'mousedown';
const OPTION_CLS = `${attrPrefix}-option`;
const OPTION_FOCUSED_CLS = `${OPTION_CLS}-focused`;
const OPTION_SELECTED_CLS = `${OPTION_CLS}-selected`;
const OPTION_DISABLED_CLS = `${OPTION_CLS}-disabled`;
const CARET_ICON_CLS = `${attrPrefix}-caret-icon`;
const CLEAR_ICON_CLS = `${attrPrefix}-clear-icon`;
const AUTOSIZE_INPUT_CLS = `${attrPrefix}-autosize-input`;
const MENU_CONTAINER_CLS = `${attrPrefix}-menu-container`;
const SELECT_CONTAINER_CLS = `${attrPrefix}-select-container`;
const CONTROL_CONTAINER_CLS = `${attrPrefix}-control-container`;
const CLEAR_ICON_TESTID = isNotProd ? CLEAR_ICON_CLS : undefined;
const CARET_ICON_TESTID = isNotProd ? CARET_ICON_CLS : undefined;
const AUTOSIZE_INPUT_TESTID = isNotProd ? AUTOSIZE_INPUT_CLS : undefined;
const MENU_CONTAINER_TESTID = isNotProd ? MENU_CONTAINER_CLS : undefined;
const CLEAR_ICON_MV_TESTID = isNotProd ? `${CLEAR_ICON_CLS}-mv` : undefined;
const SELECT_CONTAINER_TESTID = isNotProd ? SELECT_CONTAINER_CLS : undefined;
const CONTROL_CONTAINER_TESTID = isNotProd ? CONTROL_CONTAINER_CLS : undefined;

const Option = React.memo(({
  index,
  style,
  data: {
    menuOptions,
    selectOption,
    renderOptionLabel,
    focusedOptionIndex
  }
}) => {
  const {
    data,
    value,
    label,
    isDisabled,
    isSelected
  } = menuOptions[index];
  const className = OPTION_CLS + (isDisabled ? ' ' + OPTION_DISABLED_CLS : '') + (isSelected ? ' ' + OPTION_SELECTED_CLS : '') + (index === focusedOptionIndex ? ' ' + OPTION_FOCUSED_CLS : '');
  return React.createElement("div", {
    style: style,
    className: className,
    onClick: !isDisabled ? () => selectOption({
      data,
      value,
      label
    }, isSelected) : undefined
  }, renderOptionLabel(data));
}, areEqual);
Option.displayName = 'Option';

const NoOptionsMsg = styled.div`
  text-align: center;
  ${({
  theme: {
    noOptions
  }
}) => `
    color: ${noOptions.color};
    margin: ${noOptions.margin};
    padding: ${noOptions.padding};
    font-size: ${noOptions.fontSize};
  `}
`;
const Menu = React.forwardRef(({
  width,
  itemSize,
  maxHeight,
  menuOptions,
  selectOption,
  noOptionsMsg,
  overscanCount,
  renderOptionLabel,
  focusedOptionIndex
}, ref) => {
  const itemData = useMemo(() => ({
    menuOptions,
    selectOption,
    renderOptionLabel,
    focusedOptionIndex
  }), [menuOptions, focusedOptionIndex, selectOption, renderOptionLabel]);
  return React.createElement(Fragment, null, React.createElement(FixedSizeList, {
    ref: ref,
    width: width,
    itemSize: itemSize,
    itemData: itemData,
    overscanCount: overscanCount,
    itemCount: menuOptions.length,
    height: Math.min(maxHeight, menuOptions.length * itemSize)
  }, Option), !isArrayWithLength(menuOptions) && React.createElement(NoOptionsMsg, null, noOptionsMsg));
});
Menu.displayName = 'Menu';

const MultiValueWrapper = styled.div`
  min-width: 0;
  display: flex;
  ${FADE_IN_ANIMATION_CSS}
  ${({
  theme: {
    multiValue
  }
}) => `
    margin: ${multiValue.margin};
    border-radius: ${multiValue.borderRadius};
    background-color: ${multiValue.backgroundColor};
  `}
`;
const Label = styled.div`
  overflow: hidden;
  white-space: nowrap;
  text-overflow: ellipsis;
  ${({
  theme: {
    multiValue: {
      label
    }
  }
}) => `
    padding: ${label.padding};
    font-size: ${label.fontSize};
    border-radius: ${label.borderRadius};
  `}
`;
const Clear = styled.div`
  display: flex;
  ${({
  isFocused,
  theme: {
    color,
    multiValue: {
      clear
    }
  }
}) => `
    padding: ${clear.padding};
    font-size: ${clear.fontSize};
    transition: ${clear.transition};
    align-items: ${clear.alignItems};
    font-weight: ${clear.fontWeight};
    border-radius: ${clear.borderRadius};
    background-color: ${isFocused ? color.dangerLight : 'transparent'};
    :hover {
      color: ${color.danger};
      background-color: ${color.dangerLight};
    }
  `}
`;

const MultiValue = ({
  data,
  value,
  isFocused,
  renderOptionLabel,
  removeSelectedOption
}) => React.createElement(MultiValueWrapper, null, React.createElement(Label, null, renderOptionLabel(data)), React.createElement(Clear, {
  "aria-hidden": 'true',
  isFocused: isFocused,
  "data-testid": CLEAR_ICON_MV_TESTID,
  onTouchEnd: e => removeSelectedOption(value, e),
  onMouseDown: e => removeSelectedOption(value, e)
}, "X"));

const SingleValue = styled.div`
  overflow: hidden;
  position: absolute;
  white-space: nowrap;
  box-sizing: border-box;
  text-overflow: ellipsis;
  max-width: calc(100% - 0.5rem);
`;
const Placeholder = styled.div`
  overflow: hidden;
  position: absolute;
  white-space: nowrap;
  box-sizing: border-box;
  text-overflow: ellipsis;
  color: ${({
  theme
}) => theme.color.placeholder};
`;
const Value = React.memo(({
  isMulti,
  inputValue,
  placeholder,
  selectedOption,
  focusedMultiValue,
  renderOptionLabel,
  removeSelectedOption
}) => {
  if (inputValue && (!isMulti || isMulti && !isArrayWithLength(selectedOption))) {
    return null;
  }

  if (!isArrayWithLength(selectedOption)) {
    return React.createElement(Placeholder, null, placeholder);
  }

  if (!isMulti) {
    const {
      data
    } = selectedOption[0];
    return React.createElement(SingleValue, null, renderOptionLabel(data));
  }

  return React.createElement(Fragment, null, selectedOption.map(({
    data,
    value
  }) => React.createElement(MultiValue, {
    key: value,
    data: data,
    value: value,
    renderOptionLabel: renderOptionLabel,
    isFocused: value === focusedMultiValue,
    removeSelectedOption: removeSelectedOption
  })));
});
Value.displayName = 'Value';

const INPUT_MIN_WIDTH_PX = 2;
const WRAPPER_DIV_STYLE = {
  display: 'inline-block'
};
const STATIC_ATTRIBUTES = {
  type: 'text',
  spellCheck: false,
  autoCorrect: 'off',
  autoComplete: 'off',
  autoCapitalize: 'none',
  'aria-autocomplete': 'list',
  'data-testid': AUTOSIZE_INPUT_TESTID
};
const SizerDiv = styled.div`
  top: 0;
  left: 0;
  height: 0;
  overflow: scroll;
  white-space: pre;
  position: absolute;
  visibility: hidden;
  font-size: inherit;
  font-weight: inherit;
  font-family: inherit;
`;
const Input = styled.input`
  border: 0;
  outline: 0;
  padding: 0;
  cursor: text;
  background: 0;
  color: inherit;
  font-size: inherit;
  font-weight: inherit;
  font-family: inherit;
  box-sizing: content-box;

  :read-only {
    opacity: 0;
    cursor: default;
  }

  ${() => isEdgeOrIE() && `::-ms-clear { display: none; }`}
`;
const AutosizeInput = React.memo(React.forwardRef(({
  id,
  onBlur,
  onFocus,
  readOnly,
  onChange,
  ariaLabel,
  inputValue,
  addClassNames,
  ariaLabelledBy
}, ref) => {
  const sizerRef = useRef(null);
  const [inputWidth, setInputWidth] = useState(INPUT_MIN_WIDTH_PX);
  useEffect(() => {
    if (sizerRef.current) {
      setInputWidth(sizerRef.current.scrollWidth + INPUT_MIN_WIDTH_PX);
    }
  }, [inputValue]);
  const inputAttributes = Object.assign({}, STATIC_ATTRIBUTES, {
    'aria-label': ariaLabel,
    'aria-labelledby': ariaLabelledBy,
    style: {
      width: inputWidth
    }
  });
  return React.createElement("div", {
    style: WRAPPER_DIV_STYLE
  }, React.createElement(Input, Object.assign({
    id: id,
    ref: ref,
    onBlur: onBlur,
    onFocus: onFocus,
    value: inputValue,
    readOnly: readOnly
  }, inputAttributes, {
    onChange: !readOnly ? onChange : undefined,
    className: addClassNames ? AUTOSIZE_INPUT_CLS : undefined
  })), React.createElement(SizerDiv, {
    ref: sizerRef
  }, inputValue));
}));
AutosizeInput.displayName = 'AutosizeInput';

const BOUNCE_KEYFRAMES = keyframes`
  0%, 80%, 100% {
    transform: scale(0);
  } 40% {
    transform: scale(1.0);
  }
`;
const StyledLoadingDots = styled.div`
  display: flex;
  align-self: center;
  text-align: center;
  margin-right: 0.25rem;
  padding: ${({
  theme
}) => theme.loader.padding};

  > div {
    border-radius: 100%;
    display: inline-block;
    width: ${({
  theme
}) => theme.loader.size};
    height: ${({
  theme
}) => theme.loader.size};
    background-color: ${({
  theme
}) => theme.loader.color};
    animation: ${BOUNCE_KEYFRAMES} 1.19s ease-in-out infinite;

    :nth-of-type(1) {
      animation-delay: -0.272s;
    }

    :nth-of-type(2) {
      animation-delay: -0.136s;
    }
  }
`;

const LoadingDots = () => React.createElement(StyledLoadingDots, {
  "aria-hidden": 'true'
}, React.createElement("div", null), React.createElement("div", null), React.createElement("div", null));

const IndicatorIconsWrapper = styled.div`
  display: flex;
  flex-shrink: 0;
  align-items: center;
  align-self: stretch;
  box-sizing: border-box;
`;
const IndicatorIcon = styled.div`
  display: flex;
  box-sizing: border-box;
  ${({
  theme: {
    icon
  }
}) => `
    color: ${icon.color};
    padding: ${icon.padding};
    :hover {
      color: ${icon.hoverColor};
    }
  `}
`;
const Clear$1 = styled.div`
  overflow: hidden;
  ${FADE_IN_ANIMATION_CSS}
  ${({
  theme: {
    icon: {
      clear
    }
  }
}) => `
    font-size: ${clear.fontSize};
    transition: ${clear.transition};
    font-weight: ${clear.fontWeight};
    font-family: ${clear.fontFamily};
  `}
`;
const Caret = styled.div`
  ${({
  theme: {
    icon: {
      caret
    }
  }
}) => `
    transition: ${caret.transition};
    border-top: ${caret.size} dashed;
    border-left: ${caret.size} solid transparent;
    border-right: ${caret.size} solid transparent;
  `}

  ${({
  menuOpen,
  isInvalid,
  theme: {
    color
  }
}) => menuOpen && `
      transform: rotate(180deg);
      color: ${isInvalid ? color.danger : color.caretActive || color.primary};
    `}
`;
const Separator = styled.div`
  width: 1px;
  margin: 0.5rem 0;
  align-self: stretch;
  box-sizing: border-box;
  background-color: ${({
  theme
}) => theme.color.iconSeparator || theme.color.border};
`;
const IndicatorIcons = React.memo(({
  menuOpen,
  clearIcon,
  caretIcon,
  isInvalid,
  showClear,
  isLoading,
  loadingNode,
  addClassNames,
  onCaretMouseDown,
  onClearMouseDown
}) => React.createElement(IndicatorIconsWrapper, null, showClear && !isLoading && React.createElement(IndicatorIcon, {
  "aria-hidden": 'true',
  onTouchEnd: onClearMouseDown,
  onMouseDown: onClearMouseDown,
  "data-testid": CLEAR_ICON_TESTID
}, clearIcon || React.createElement(Clear$1, {
  className: addClassNames ? CLEAR_ICON_CLS : undefined
}, "X")), isLoading && (loadingNode || React.createElement(LoadingDots, null)), React.createElement(Separator, null), React.createElement(IndicatorIcon, {
  "aria-hidden": 'true',
  onTouchEnd: onCaretMouseDown,
  onMouseDown: onCaretMouseDown,
  "data-testid": CARET_ICON_TESTID
}, caretIcon || React.createElement(Caret, {
  menuOpen: menuOpen,
  isInvalid: isInvalid,
  className: addClassNames ? CARET_ICON_CLS : undefined
}))));
IndicatorIcons.displayName = 'IndicatorIcons';

const A11yText = styled.span`
  border: 0;
  padding: 0;
  width: 1px;
  height: 1px;
  margin: -1px;
  overflow: hidden;
  position: absolute;
  white-space: nowrap;
  clip: rect(0 0 0 0);
  clip-path: inset(50%);
`;

const AriaLiveRegion = ({
  menuOpen,
  isFocused,
  ariaLabel,
  inputValue,
  optionCount,
  isSearchable,
  focusedOption,
  selectedOption
}) => {
  if (!isFocused) {
    return null;
  }

  const {
    label: focusedOptionLabel,
    index: focusedOptionIndex,
    isDisabled: isFocusedOptionDisabled
  } = focusedOption;
  const selectedOptionLabel = isArrayWithLength(selectedOption) ? selectedOption.map(x => x.label).join(' ') : 'N/A';
  const selectedOptionMsg = `Selected option: ${selectedOptionLabel}`;
  const optionsMsg = `${optionCount} result(s) available${inputValue ? ` for search input ${inputValue}` : ''}.`;
  const focusedMsg = `Focused option: ${focusedOptionLabel || 'N/A'}${isFocusedOptionDisabled ? ' - disabled' : ''}, ${focusedOptionIndex + 1} of ${optionCount}.`;
  const menuMsg = menuOpen ? 'Use Up and Down arrow keys to choose options, press Enter or Tab to select the currently focused option, press Escape to close the menu.' : `${ariaLabel || 'Select'} is focused${isSearchable ? ', type to filter options' : ''}, press Down arrow key to open the menu.`;
  return React.createElement(A11yText, {
    "aria-live": 'polite'
  }, React.createElement("p", {
    id: 'aria-selection-event'
  }, selectedOptionMsg), React.createElement("p", {
    id: 'aria-context'
  }, `${focusedMsg} ${optionsMsg} ${menuMsg}`));
};

const SelectWrapper = styled.div`
  position: relative;
  box-sizing: border-box;
  ${({
  theme
}) => theme.color.textColor ? `color: ${theme.color.textColor};` : ''}
  ${({
  theme
}) => theme.select.fontSize ? `font-size: ${theme.select.fontSize};` : ''}
`;
const ValueWrapper = styled.div`
  flex: 1 1 0%;
  display: flex;
  flex-wrap: wrap;
  overflow: hidden;
  position: relative;
  align-items: center;
  box-sizing: border-box;
  padding: ${({
  theme
}) => theme.control.padding};
`;
const ControlWrapper = styled.div`
  outline: 0;
  display: flex;
  flex-wrap: wrap;
  cursor: default;
  position: relative;
  align-items: center;
  box-sizing: border-box;
  justify-content: space-between;
  ${({
  isDisabled,
  isFocused,
  isInvalid,
  theme: {
    control,
    color
  }
}) => `
    min-height: ${control.minHeight};
    transition: ${control.transition};
    border-style: ${control.borderStyle};
    border-width: ${control.borderWidth};
    border-radius: ${control.borderRadius};
    border-color: ${isInvalid ? color.danger : isFocused ? control.focusedBorderColor : color.border};
    ${isDisabled ? 'pointer-events: none;' : ''}
    ${control.height ? `height: ${control.height};` : ''}
    ${control.backgroundColor || isDisabled ? `background-color: ${isDisabled ? color.disabled : control.backgroundColor};` : ''}
    ${isFocused ? `box-shadow: ${control.boxShadow} ${isInvalid ? color.dangerLight : control.boxShadowColor};` : ''}
  `}
`;
const MenuWrapper = styled.div`
  z-index: 999;
  cursor: default;
  position: absolute;
  ${FADE_IN_ANIMATION_CSS}
  ${({
  hideMenu,
  theme: {
    menu
  }
}) => `
    width: ${menu.width};
    margin: ${menu.margin};
    padding: ${menu.padding};
    box-shadow: ${menu.boxShadow};
    border-radius: ${menu.borderRadius};
    background-color: ${menu.backgroundColor};
    ${hideMenu ? 'display: none;' : ''}
  `}

  .${OPTION_CLS} {
    display: block;
    overflow: hidden;
    user-select: none;
    white-space: nowrap;
    text-overflow: ellipsis;
    -webkit-tap-highlight-color: transparent;
    ${({
  theme: {
    menu: {
      option
    }
  }
}) => `
      padding: ${option.padding};
      text-align: ${option.textAlign};

      &.${OPTION_FOCUSED_CLS},
      &:hover:not(.${OPTION_DISABLED_CLS}):not(.${OPTION_SELECTED_CLS}) {
        background-color: ${option.focusedBgColor};
      }

      &.${OPTION_SELECTED_CLS} {
        color: ${option.selectedColor};
        background-color: ${option.selectedBgColor};
      }

      &.${OPTION_DISABLED_CLS} {
        opacity: 0.35;
      }
    `}
  }
`;
const Select = React.forwardRef(({
  isMulti,
  inputId,
  selectId,
  autoFocus,
  isLoading,
  onKeyDown,
  clearIcon,
  caretIcon,
  isInvalid,
  ariaLabel,
  menuWidth,
  isDisabled,
  inputDelay,
  onMenuOpen,
  onMenuClose,
  onInputBlur,
  isClearable,
  themeConfig,
  loadingNode,
  onInputFocus,
  initialValue,
  addClassNames,
  ariaLabelledBy,
  onOptionChange,
  getOptionLabel,
  getOptionValue,
  openMenuOnFocus,
  isAriaLiveEnabled,
  menuOverscanCount,
  blurInputOnSelect,
  renderOptionLabel,
  menuScrollDuration,
  filterIgnoreAccents,
  hideSelectedOptions,
  getIsOptionDisabled,
  getFilterOptionString,
  isSearchable = true,
  openMenuOnClick = true,
  filterIgnoreCase = true,
  tabSelectsOption = true,
  closeMenuOnSelect = true,
  scrollMenuIntoView = true,
  backspaceClearsValue = true,
  options = OPTIONS_DEFAULT,
  placeholder = PLACEHOLDER_DEFAULT,
  filterMatchFrom = FilterMatchEnum.ANY,
  noOptionsMsg = NO_OPTIONS_MSG_DEFAULT,
  menuItemSize = MENU_ITEM_SIZE_DEFAULT,
  menuMaxHeight = MENU_MAX_HEIGHT_DEFAULT
}, ref) => {
  const prevMenuOptionsCount = useRef();
  const listRef = useRef(null);
  const menuRef = useRef(null);
  const inputRef = useRef(null);
  const [inputValue, setInputValue] = useState('');
  const [menuOpen, setMenuOpen] = useState(false);
  const [isFocused, setIsFocused] = useState(false);
  const [focusedOption, setFocusedOption] = useState(FOCUSED_OPTION_DEFAULT);
  const [focusedMultiValue, setFocusedMultiValue] = useState(FOCUSED_MULTI_DEFAULT);
  const {
    data: focusedOptionData,
    value: focusedOptionValue,
    label: focusedOptionLabel,
    index: focusedOptionIndex,
    isDisabled: isFocusedOptionDisabled,
    isSelected: isFocusedOptionSelected
  } = focusedOption;
  const theme = useMemo(() => {
    return !isPlainObject(themeConfig) ? Object.assign({}, DefaultThemeObj) : mergeDeep(DefaultThemeObj, themeConfig);
  }, [themeConfig]);
  const getOptionLabelCB = useMemo(() => {
    return getOptionLabel || (data => data.label);
  }, [getOptionLabel]);
  const getOptionValueCB = useMemo(() => {
    return getOptionValue || (data => data.value);
  }, [getOptionValue]);
  const renderOptionLabelCB = useMemo(() => {
    return renderOptionLabel || getOptionLabelCB;
  }, [renderOptionLabel, getOptionLabelCB]);
  const debouncedInputValue = useDebounce(inputValue, inputDelay);
  const [selectedOption, setSelectedOption] = useState(() => normalizeValue(initialValue, getOptionValueCB, getOptionLabelCB));
  const menuHeight = useMenuHeight(menuRef, menuOpen, menuMaxHeight, menuScrollDuration, scrollMenuIntoView, onMenuOpen, onMenuClose);
  const hideSelectedOptionsOrDefault = typeof hideSelectedOptions !== 'boolean' ? !!isMulti : hideSelectedOptions;
  const menuOptions = useMenuOptions(options, debouncedInputValue, filterMatchFrom, hideSelectedOptionsOrDefault, selectedOption, getOptionValueCB, getOptionLabelCB, getIsOptionDisabled, getFilterOptionString, filterIgnoreCase, filterIgnoreAccents);

  const blurInput = () => {
    inputRef.current && inputRef.current.blur();
  };

  const focusInput = () => {
    inputRef.current && inputRef.current.focus();
  };

  const scrollToItemIndex = index => {
    listRef.current && listRef.current.scrollToItem(index);
  };

  const removeSelectedOption = useCallback((value, e) => {
    if (e) {
      e.stopPropagation();
      e.type === MOUSE_DOWN_EVENT_TYPE && e.preventDefault();
    }

    setSelectedOption(prevSelectedOption => prevSelectedOption.filter(x => x.value !== value));
  }, []);
  const openMenuAndFocusOption = useCallback(position => {
    if (!isArrayWithLength(menuOptions)) {
      setMenuOpen(true);
      return;
    }

    const selectedIndex = !isMulti ? menuOptions.findIndex(option => option.isSelected) : -1;
    const index = selectedIndex > -1 ? selectedIndex : position === OptionIndexEnum.FIRST ? 0 : menuOptions.length - 1;
    setMenuOpen(true);
    setFocusedOption(Object.assign({
      index
    }, menuOptions[index]));
    scrollToItemIndex(index);
  }, [isMulti, menuOptions]);
  const selectOption = useCallback((option, isSelected) => {
    if (isSelected) {
      isMulti && removeSelectedOption(option.value);
    } else {
      setSelectedOption(prevSelectedOption => !isMulti ? [option] : [...prevSelectedOption, option]);
    }

    const blurInputOnSelectOrDefault = typeof blurInputOnSelect === 'boolean' ? blurInputOnSelect : isTouchDevice();

    if (blurInputOnSelectOrDefault) {
      blurInput();
    } else if (closeMenuOnSelect) {
      setMenuOpen(false);
      setInputValue('');
    }
  }, [isMulti, closeMenuOnSelect, blurInputOnSelect, removeSelectedOption]);
  useImperativeHandle(ref, () => ({
    blur: blurInput,
    focus: focusInput,
    clearValue: () => {
      setSelectedOption(SELECTED_OPTION_DEFAULT);
      setFocusedOption(FOCUSED_OPTION_DEFAULT);
    },
    setValue: option => {
      const validatedSelectedOption = validateSetValueParam(option, menuOptions, getOptionValueCB);
      setSelectedOption(validatedSelectedOption);
    },
    toggleMenu: state => {
      if (state === true || state === undefined && !menuOpen) {
        !isFocused && focusInput();
        openMenuAndFocusOption(OptionIndexEnum.FIRST);
      } else if (menuOpen) {
        blurInput();
      }
    }
  }));
  useEffect(() => {
    autoFocus && focusInput();
  }, [autoFocus]);
  useEffect(() => {
    if (isFocused && openMenuOnFocus) {
      openMenuAndFocusOption(OptionIndexEnum.FIRST);
    }
  }, [isFocused, openMenuOnFocus, openMenuAndFocusOption]);
  useUpdateEffect(() => {
    if (onOptionChange) {
      const normalizedOptionValue = isMulti ? selectedOption.map(x => x.data) : isArrayWithLength(selectedOption) ? selectedOption[0].data : ON_CHANGE_SINGLE_VALUE_DEFAULT;
      onOptionChange(normalizedOptionValue);
    }
  }, [isMulti, selectedOption, onOptionChange]);
  useEffect(() => {
    if (typeof prevMenuOptionsCount.current === 'number' && !isArrayWithLength(menuOptions)) {
      setFocusedOption(FOCUSED_OPTION_DEFAULT);
    } else if (menuOptions.length === 1 || !!menuOptions.length && (menuOptions.length !== options.length || prevMenuOptionsCount.current === 0)) {
      setFocusedOption(Object.assign({
        index: 0
      }, menuOptions[0]));
      scrollToItemIndex(0);
    }

    prevMenuOptionsCount.current = menuOptions.length;
  }, [options, menuOptions]);

  const selectOptionFromFocused = () => {
    if (!focusedOptionData || isFocusedOptionDisabled) {
      return;
    }

    const newOption = {
      data: focusedOptionData,
      value: focusedOptionValue,
      label: focusedOptionLabel
    };
    selectOption(newOption, isFocusedOptionSelected);
  };

  const focusValueOnArrowKey = direction => {
    if (!isArrayWithLength(selectedOption)) {
      return;
    }

    let nextFocusedIndex = -1;
    const lastValuesIndex = selectedOption.length - 1;
    const curFocusedIndex = focusedMultiValue ? selectedOption.findIndex(option => option.value === focusedMultiValue) : -1;

    if (direction === ValueIndexEnum.NEXT) {
      nextFocusedIndex = curFocusedIndex > -1 && curFocusedIndex < lastValuesIndex ? curFocusedIndex + 1 : -1;
    } else {
      nextFocusedIndex = curFocusedIndex !== 0 ? curFocusedIndex === -1 ? lastValuesIndex : curFocusedIndex - 1 : 0;
    }

    const nextFocusedVal = nextFocusedIndex === -1 ? FOCUSED_MULTI_DEFAULT : selectedOption[nextFocusedIndex].value;
    focusedOptionData && setFocusedOption(FOCUSED_OPTION_DEFAULT);
    nextFocusedVal !== focusedMultiValue && setFocusedMultiValue(nextFocusedVal);
  };

  const focusOptionOnArrowKey = direction => {
    if (!isArrayWithLength(menuOptions)) {
      return;
    }

    const index = direction === OptionIndexEnum.DOWN ? (focusedOptionIndex + 1) % menuOptions.length : focusedOptionIndex > 0 ? focusedOptionIndex - 1 : menuOptions.length - 1;
    focusedMultiValue && setFocusedMultiValue(FOCUSED_MULTI_DEFAULT);
    setFocusedOption(Object.assign({
      index
    }, menuOptions[index]));
    scrollToItemIndex(index);
  };

  const handleOnKeyDown = e => {
    if (isDisabled) {
      return;
    }

    if (onKeyDown) {
      onKeyDown(e);

      if (e.defaultPrevented) {
        return;
      }
    }

    switch (e.key) {
      case 'ArrowDown':
        if (menuOpen) {
          focusOptionOnArrowKey(OptionIndexEnum.DOWN);
        } else {
          openMenuAndFocusOption(OptionIndexEnum.FIRST);
        }

        break;

      case 'ArrowUp':
        if (menuOpen) {
          focusOptionOnArrowKey(OptionIndexEnum.UP);
        } else {
          openMenuAndFocusOption(OptionIndexEnum.LAST);
        }

        break;

      case 'ArrowLeft':
        if (!isMulti || inputValue) {
          return;
        }

        focusValueOnArrowKey(ValueIndexEnum.PREVIOUS);
        break;

      case 'ArrowRight':
        if (!isMulti || inputValue) {
          return;
        }

        focusValueOnArrowKey(ValueIndexEnum.NEXT);
        break;

      case ' ':
        if (inputValue) {
          return;
        } else if (!menuOpen) {
          openMenuAndFocusOption(OptionIndexEnum.FIRST);
          break;
        } else if (!focusedOptionData) {
          return;
        }

        selectOptionFromFocused();
        break;

      case 'Enter':
        if (menuOpen && e.keyCode !== IME_KEY_CODE) {
          selectOptionFromFocused();
        }

        break;

      case 'Escape':
        if (menuOpen) {
          setMenuOpen(false);
          setInputValue('');
        }

        break;

      case 'Tab':
        if (!menuOpen || !tabSelectsOption || !focusedOptionData || e.shiftKey) {
          return;
        }

        selectOptionFromFocused();
        break;

      case 'Delete':
      case 'Backspace':
        if (inputValue) {
          return;
        }

        if (focusedMultiValue) {
          const clearFocusedIndex = selectedOption.findIndex(option => option.value === focusedMultiValue);
          const nexFocusedMultiValue = clearFocusedIndex > -1 && clearFocusedIndex < selectedOption.length - 1 ? selectedOption[clearFocusedIndex + 1].value : FOCUSED_MULTI_DEFAULT;
          removeSelectedOption(focusedMultiValue);
          setFocusedMultiValue(nexFocusedMultiValue);
        } else {
          if (!backspaceClearsValue) {
            return;
          }

          if (isArrayWithLength(selectedOption)) {
            if (isMulti) {
              const {
                value
              } = selectedOption[selectedOption.length - 1];
              removeSelectedOption(value);
            } else if (isClearable) {
              setSelectedOption(SELECTED_OPTION_DEFAULT);
            }
          }
        }

        break;

      default:
        return;
    }

    e.preventDefault();
  };

  const handleOnControlMouseDown = e => {
    if (isDisabled) {
      return;
    }

    if (!isFocused) {
      focusInput();
    }

    if (!menuOpen) {
      openMenuOnClick && openMenuAndFocusOption(OptionIndexEnum.FIRST);
    } else if (e.currentTarget.tagName !== INPUT_TAG_NAME) {
      setMenuOpen(false);
      inputValue && setInputValue('');
    }

    if (e.currentTarget.tagName !== INPUT_TAG_NAME) {
      e.preventDefault();
    }
  };

  const handleOnMenuMouseDown = e => {
    e.stopPropagation();
    e.preventDefault();
    focusInput();
  };

  const handleOnInputBlur = useCallback(e => {
    onInputBlur && onInputBlur(e);
    setIsFocused(false);
    setMenuOpen(false);
    setInputValue('');
  }, [onInputBlur]);
  const handleOnInputFocus = useCallback(e => {
    onInputFocus && onInputFocus(e);
    setIsFocused(true);
  }, [onInputFocus]);
  const handleOnInputChange = useCallback(e => {
    setMenuOpen(true);
    setInputValue(e.currentTarget.value || '');
  }, []);
  const handleOnClearMouseDown = useCallback(e => {
    e.stopPropagation();
    e.type === MOUSE_DOWN_EVENT_TYPE && e.preventDefault();
    setSelectedOption(SELECTED_OPTION_DEFAULT);
    focusInput();
  }, []);
  const handleOnCaretMouseDown = useCallback(e => {
    e.stopPropagation();
    e.type === MOUSE_DOWN_EVENT_TYPE && e.preventDefault();
    focusInput();

    if (menuOpen) {
      setMenuOpen(false);
    } else {
      openMenuAndFocusOption(OptionIndexEnum.FIRST);
    }
  }, [menuOpen, openMenuAndFocusOption]);
  return React.createElement(ThemeProvider, {
    theme: theme
  }, React.createElement(SelectWrapper, {
    id: selectId,
    onKeyDown: handleOnKeyDown,
    "data-testid": SELECT_CONTAINER_TESTID,
    className: addClassNames ? SELECT_CONTAINER_CLS : undefined
  }, React.createElement(ControlWrapper, {
    isInvalid: isInvalid,
    isFocused: isFocused,
    isDisabled: isDisabled,
    onTouchEnd: handleOnControlMouseDown,
    onMouseDown: handleOnControlMouseDown,
    "data-testid": CONTROL_CONTAINER_TESTID,
    className: addClassNames ? CONTROL_CONTAINER_CLS : undefined
  }, React.createElement(ValueWrapper, null, React.createElement(Value, {
    isMulti: isMulti,
    inputValue: inputValue,
    placeholder: placeholder,
    selectedOption: selectedOption,
    focusedMultiValue: focusedMultiValue,
    renderOptionLabel: renderOptionLabelCB,
    removeSelectedOption: removeSelectedOption
  }), React.createElement(AutosizeInput, {
    id: inputId,
    ref: inputRef,
    ariaLabel: ariaLabel,
    inputValue: inputValue,
    onBlur: handleOnInputBlur,
    onFocus: handleOnInputFocus,
    addClassNames: addClassNames,
    onChange: handleOnInputChange,
    ariaLabelledBy: ariaLabelledBy,
    readOnly: isDisabled || !isSearchable || !!focusedMultiValue
  })), React.createElement(IndicatorIcons, {
    menuOpen: menuOpen,
    clearIcon: clearIcon,
    caretIcon: caretIcon,
    isInvalid: isInvalid,
    isLoading: isLoading,
    loadingNode: loadingNode,
    addClassNames: addClassNames,
    onClearMouseDown: handleOnClearMouseDown,
    showClear: !!(isClearable && !isDisabled && isArrayWithLength(selectedOption)),
    onCaretMouseDown: !isDisabled && !openMenuOnClick ? handleOnCaretMouseDown : undefined
  })), React.createElement(MenuWrapper, {
    ref: menuRef,
    hideMenu: !menuOpen,
    onMouseDown: handleOnMenuMouseDown,
    "data-testid": MENU_CONTAINER_TESTID,
    className: addClassNames ? MENU_CONTAINER_CLS : undefined
  }, React.createElement(Menu, {
    ref: listRef,
    maxHeight: menuHeight,
    itemSize: menuItemSize,
    menuOptions: menuOptions,
    noOptionsMsg: noOptionsMsg,
    selectOption: selectOption,
    overscanCount: menuOverscanCount,
    width: menuWidth || theme.menu.width,
    focusedOptionIndex: focusedOptionIndex,
    renderOptionLabel: renderOptionLabelCB
  })), isAriaLiveEnabled && React.createElement(AriaLiveRegion, {
    menuOpen: menuOpen,
    isFocused: isFocused,
    ariaLabel: ariaLabel,
    inputValue: inputValue,
    isSearchable: isSearchable,
    focusedOption: focusedOption,
    selectedOption: selectedOption,
    optionCount: menuOptions.length
  })));
});
Select.displayName = 'Select';

export { FilterMatchEnum, Select };
